@@ -44,7 +44,8 @@ def handleBikeshedInclude(el: t.ElementT, doc: t.SpecT) -> None:
             return
         doc.recordDependencies(includedInputSource)
         try:
-            lines = includedInputSource.read().rawLines
+            text = includedInputSource.read().content
+            nodes = list(h.initialDocumentParse(text))
         except Exception as err:
             m.die(f"Couldn't find include file '{path}'. Error was:\n{err}", el=el)
             h.removeNode(el)
@@ -53,7 +54,7 @@ def handleBikeshedInclude(el: t.ElementT, doc: t.SpecT) -> None:
         # can't use just path, because they're relative; including "foo/bar.txt" might use "foo/bar.txt" further nested
         # can't use just content, because then you can't do the same thing twice.
         # combined does a good job unless you purposely pervert it
-        hash = hashlib.md5((path + "".join(lines)).encode("ascii", "xmlcharrefreplace")).hexdigest()
+        hash = hashlib.md5((path + text).encode("ascii", "xmlcharrefreplace")).hexdigest()
         if el.get("hash"):
             hashAttr = el.get("hash")
             assert hashAttr is not None
@@ -70,7 +71,8 @@ def handleBikeshedInclude(el: t.ElementT, doc: t.SpecT) -> None:
             m.die("Nesting depth > 100, literally wtf are you doing.", el=el)
             h.removeNode(el)
             return
-        lines = datablocks.transformDataBlocks(doc, lines)
+        nodes = datablocks.transformDataBlocks(doc, nodes)
+        lines = "".join(str(x) for x in nodes).split("\n")
         lines = markdown.parse(lines, doc.md.indent, opaqueElements=doc.md.opaqueElements)
         text = "".join(lines)
         text = doc.fixText(text, moreMacros=macros)
@@ -157,21 +159,22 @@ def handleRawInclude(el: t.ElementT, doc: t.SpecT) -> None:
     h.replaceNode(el, *subtree)
 
 
-RangeItem: t.TypeAlias = "t.List[t.Optional[int]]"
-# UGH I DON'T UNDERSTAND THE ERROR HERE
-# fuck it, replace this with a tiny dataclass anyway
+if t.TYPE_CHECKING:
+    RangeItem: t.TypeAlias = list[int | None]
+    # UGH I DON'T UNDERSTAND THE ERROR HERE
+    # fuck it, replace this with a tiny dataclass anyway
 
 
-def parseRangeString(rangeStr: str) -> t.List[RangeItem]:
+def parseRangeString(rangeStr: str) -> list[RangeItem]:
     rangeStr = re.sub(r"\s*", "", rangeStr)
     return [_f for _f in (parseSingleRange(x) for x in rangeStr.split(",")) if _f is not None]
 
 
-def parseSingleRange(item: str) -> t.Optional[RangeItem]:
+def parseSingleRange(item: str) -> RangeItem | None:
     if "-" in item:
         # Range, format of DDD-DDD
-        low: t.Optional[int]
-        high: t.Optional[int]
+        low: int | None
+        high: int | None
         lowStr, _, highStr = item.partition("-")
         if lowStr == "*":
             low = None
